import { CompanyCreateEffects } from './ngrx/companies/effects.company.create';
import { CompanyUpdateDeleteEffects } from './ngrx/companies/effects.company.update.delete';
import { CompanyGetSearchEffects } from './ngrx/companies/effects.company.get';
import { BaseNavMenuComponent } from './pages/base-nav-menu/base-nav-menu.component';
import { FormsModule } from '@angular/forms';
import { AccueilModule } from './pages/accueil/accueil.module';
import { CompaniesManagerModule } from './pages/companies-manager/companies-manager.module';
import { ProjectsManagerModule } from './pages/projects-manager/projects-manager.module';
import { UsersManagerModule } from './pages/users-manager/users-manager.module';
import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { HttpClientModule } from '@angular/common/http';
import { LayoutModule } from '@angular/cdk/layout';
import { MatToolbarModule } from '@angular/material/toolbar';
import { MatButtonModule } from '@angular/material/button';
import { MatSidenavModule } from '@angular/material/sidenav';
import { MatIconModule } from '@angular/material/icon';
import { MatListModule } from '@angular/material/list';
import { MatMenuModule } from '@angular/material/menu';
import { EffectsModule } from '@ngrx/effects';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { UserCreateEffects } from './ngrx/users/effects.user.create';
import { UserGetSearchEffects } from './ngrx/users/effects.user.get';
import { UserUpdateEffects } from './ngrx/users/effects.user.update.delete';
import { userReducer } from './ngrx/users/users.reducer';
import { projectReducer } from './ngrx/projects/projects.reducer';
import { ProjectUpdateEffects } from './ngrx/projects/effects.project.update.delete';
import { ProjectCreateEffects } from './ngrx/projects/effects.project.create';
import { ProjectGetEffects } from './ngrx/projects/effects.project.get';
import { DatePipe } from '@angular/common';
import { companyReducer } from './ngrx/companies/companies.reducer';

@NgModule({
  declarations: [AppComponent, BaseNavMenuComponent],
  imports: [
    BrowserModule,
    AppRoutingModule,
    UsersManagerModule,
    ProjectsManagerModule,
    CompaniesManagerModule,
    AccueilModule,
    HttpClientModule,
    BrowserAnimationsModule,
    FormsModule,
    LayoutModule,
    MatToolbarModule,
    MatButtonModule,
    MatSidenavModule,
    MatIconModule,
    MatListModule,
    MatMenuModule,
    StoreModule.forRoot({
      userReducerKey: userReducer,
      projectReducerKey: projectReducer,
      companyReducerKey: companyReducer
    }),
    EffectsModule.forRoot([
      UserCreateEffects,
      UserGetSearchEffects,
      UserUpdateEffects,
      ProjectCreateEffects,
      ProjectGetEffects,
      ProjectUpdateEffects,
      CompanyGetSearchEffects,
      CompanyUpdateDeleteEffects,
      CompanyCreateEffects
    ]),
    StoreDevtoolsModule.instrument(),
  ],
  bootstrap: [AppComponent],
})
export class AppModule {}
