import { Action } from "@ngrx/store";
import { Company } from "src/app/shared/models/company/company.model";
import { MsgState } from "../msg.state";
import { CompanyActionUnion } from "./companies.actions";
import { CompanyActionTypes } from "./company.action.types";
import { CompanyState } from "./company.state";

/**reducer fonction to handle actions
a reducer function need initial state and an action to handle*/

const companyInitialState: CompanyState = {
  currentCompany: null,
  companiesList: [],
  msgState: MsgState.INIT_STATE,
  serverMsgError: 'no error msg'
};

export function companyReducer(state: CompanyState = companyInitialState, action: Action
) : CompanyState {

  switch (action.type) {
    // get all companies
    case CompanyActionTypes.GET_ALL_COMPANIES:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };

    case CompanyActionTypes.GET_ALL_COMPANIES_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        companiesList: (<CompanyActionUnion> action).payload
      };

    case CompanyActionTypes.GET_ALL_COMPANIES_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        serverMsgError: (<CompanyActionUnion>action).payload,
      };

    //create form to update company
    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };
    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE_SUCCESS:
      let companyToUpdate: Company = (<CompanyActionUnion>action).payload;
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
        currentCompany: companyToUpdate,
      };

    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE_ERROR:
      return {
        ...state,
        msgState: MsgState.FORM_LOAD_ERR,
        serverMsgError: (<CompanyActionUnion>action).payload,
      };

    //save form data to update company
    case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA:
      return {
        ...state,
        msgState: MsgState.UPDATE,
      };
    case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA_SUCCESS:
      let updatedCompany: Company = (<CompanyActionUnion>action).payload;
      let companiesListCopy1: Company[] = [...state.companiesList];
      companiesListCopy1 = companiesListCopy1.map((company) =>
        company.id == updatedCompany.id ? updatedCompany : company
      );

      return{
        ...state,
        msgState: MsgState.UPDATED,
        companiesList: companiesListCopy1
      }

      case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA_ERROR:
        return{
          ...state,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion>action).payload
        };

        // delete a company
      case CompanyActionTypes.DELETE_COMPANY:
        return{
          ...state,
          msgState: MsgState.DELETE
        }

      case CompanyActionTypes.DELETE_COMPANY_SUCCESS:
        let deletedCompany: Company = (<CompanyActionUnion> action).payload;
        let deletedCompanyIndex = state.companiesList.indexOf(deletedCompany);
        let companiesListCopy2: Company[] = [...state.companiesList];
        companiesListCopy2 = companiesListCopy2.splice(deletedCompanyIndex);

        return{
          ...state,
          msgState: MsgState.DELETED,
          companiesList: companiesListCopy2
        };

      case CompanyActionTypes.DELETE_COMPANY_ERROR:
        return{
          ... state,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion> action).payload
        };

      // Get company's projects
      case CompanyActionTypes.GET_COMPANY_S_PROJECTS:
        return{
          ...state,
          msgState: MsgState.LOADING
        };

      case CompanyActionTypes.GET_COMPANY_S_PROJECTS_SUCCESS:
        return{
          ...state,
          msgState: MsgState.LOADED,
          companiesList: (<CompanyActionUnion> action).payload
        };

      case CompanyActionTypes.GET_COMPANY_S_PROJECTS_ERROR:
        return{
          ...state,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion> action).payload
        };

      //create form to add company
      case CompanyActionTypes.CREATE_COMPANY_FORM_ADD:
        return{
          ...state,
          msgState: MsgState.FORM_LOADING,
        };

      case CompanyActionTypes.CREATE_COMPANY_FORM_ADD_SUCCESS:
        return{
          ...state,
          msgState: MsgState.FORM_LOADED
        };
      case CompanyActionTypes.CREATE_COMPANY_FORM_ADD_ERROR:
        return{
          ...state,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion> action).payload
        };

      // save company with form data
      case CompanyActionTypes.SAVE_COMPANY_FORM_DATA_ADD:
        return{
          ...state,
          msgState: MsgState.ADD
        };

      case CompanyActionTypes.SAVE_COMPANY_FORM_DATA_ADD_SUCCESS:
        let createdCompany: Company = (<CompanyActionUnion> action).payload;
        let companiesListCopy3 : Company[] = [...state.companiesList];
        companiesListCopy3.push(createdCompany);

        return{
          ...state,
          msgState: MsgState.ADDED,
          companiesList: companiesListCopy3
        };

      case CompanyActionTypes.SAVE_COMPANY_FORM_DATA_ADD_ERROR:
        return{
          ...state,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion> action).payload
        };

    default:
      return { ...state };
  }
}
