import { Company } from 'src/app/shared/models/company/company.model';
import { CompanyActionTypes } from './company.action.types';
import { CompanyActionUnion, CreateCompanyFormAddActionSuccess, SaveCompanyFormDataAddActionSuccess, SaveCompanyFormDataAddActionError } from './companies.actions';
import { map, Observable, mergeMap, catchError, of } from 'rxjs';
import { CompaniesServices } from './../../shared/services/companies.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Injectable } from '@angular/core';

@Injectable()
export class CompanyCreateEffects {
  constructor(
    private companyActions: Actions,
    private companiesService: CompaniesServices
  ) {}

  // create company form to add company
  createFormToAddCompanyEffect: Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
        ofType(CompanyActionTypes.CREATE_COMPANY_FORM_ADD),
        map((action: CompanyActionUnion) =>{
          return new CreateCompanyFormAddActionSuccess()
        })
      )
  );

  // save company form data
  saveCompanyFormDataAddEffect : Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
      ofType(CompanyActionTypes.SAVE_COMPANY_FORM_DATA_ADD),
      mergeMap( (action : CompanyActionUnion)  =>{
        return this.companiesService.createCompany(action.payload).pipe(
          map ( (company: Company) => new SaveCompanyFormDataAddActionSuccess(company)),
          catchError( err => of(new SaveCompanyFormDataAddActionError( err.message)))
        )
      })
    )
  );

}
