import { Injectable } from '@angular/core';
import { CompanyActionTypes } from './company.action.types';
import {
  CompanyActionUnion,
  GetAllCompaniesActionSuccess,
  GetAllCompaniesActionError,
  GetCompany_S_ProjectsSuccess,
  GetCompany_S_ProjectsError,
} from './companies.actions';
import { Observable, mergeMap, catchError, map, of } from 'rxjs';
import { CompaniesServices } from '../../shared/services/companies.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Company } from 'src/app/shared/models/company/company.model';

@Injectable()
export class CompanyGetSearchEffects {
  constructor(
    private companyActions: Actions,
    private companiesService: CompaniesServices
  ) {}

  // get all companies effect
  getAllCompaniesEffect: Observable<CompanyActionUnion> = createEffect(() =>
    this.companyActions.pipe(
      ofType(CompanyActionTypes.GET_ALL_COMPANIES),
      mergeMap(() => {
        return this.companiesService.getAllCompanies().pipe(
          map(
            (companies: Company[]) =>
              new GetAllCompaniesActionSuccess(companies)
          ),
          catchError((err) => of(new GetAllCompaniesActionError(err.message)))
        );
      })
    )
  );

  // get company's projects effect
  getCompany_s_ProjectEffect: Observable<CompanyActionUnion> = createEffect(
    () =>
      this.companyActions.pipe(
        ofType(CompanyActionTypes.GET_COMPANY_S_PROJECTS),
        mergeMap((action: CompanyActionUnion) => {
          return this.companiesService.getCompany_s_Projects().pipe(
            map(
              (company_s_ProjectsList: any) =>
                new GetCompany_S_ProjectsSuccess(company_s_ProjectsList)
            ),
            catchError((err) => of(new GetCompany_S_ProjectsError(err.message)))
          );
        })
      )
  );
}
