import { CompanyActionTypes } from './company.action.types';
import { CompanyActionUnion, CreateFormToUpdateCompanyActionSuccess, CreateFormToUpdateCompanyActionError, UpdateCompanyWithFormDataActionSuccess, UpdateCompanyWithFormDataActionError, DeleteCompanyAction, DeleteCompanyActionSuccess, DeleteCompanyActionError } from './companies.actions';
import { Observable, mergeMap, map, catchError, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { CompaniesServices } from './../../shared/services/companies.service';
import { Injectable } from '@angular/core';
import { Company } from 'src/app/shared/models/company/company.model';

@Injectable()
export class CompanyUpdateDeleteEffects {
  constructor(
    private companiesService: CompaniesServices,
    private companyActions: Actions
  ) {}

  //create form to update company effect
  createFormToUpdateCompanyEffect : Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
      ofType(CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE),
      mergeMap((action: CompanyActionUnion) =>{
        return this.companiesService.getCompanyById(action.payload.id).pipe(
          map( (company: Company) => new CreateFormToUpdateCompanyActionSuccess(company)),
          catchError(err => of(new CreateFormToUpdateCompanyActionError(err.message)))
        )
      })
    )
  );

  //update company with form data

  updateCompanyWithFormDataEffect : Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
      ofType(CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA),
      mergeMap((action: CompanyActionUnion)=>{
        return this.companiesService.updateCompany(action.payload).pipe(
          map((company: Company) => new UpdateCompanyWithFormDataActionSuccess(company)),
          catchError(err=> of(new UpdateCompanyWithFormDataActionError(err.message)))
        )
      })
    )
  );

  // delete a company
  deleteCompany: Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
      ofType(CompanyActionTypes.DELETE_COMPANY),
      mergeMap((action: CompanyActionUnion)=>{
        return this.companiesService.deleteCompany(action.payload.id).pipe(
          map(()=>new DeleteCompanyActionSuccess(action.payload)),
          catchError(err=> of(new DeleteCompanyActionError(err.message)))
        )
      })
    )
  );
}
