import { ProjectActionTypes } from './project.action.types';
import { ProjectActionUnion, CreateProjectFormAddAction, SaveProjectFormDataActionSuccess, SaveProjectFormDataActionError, CreateProjectFormAddActionSuccess } from './projects.actions';
import { map, Observable, mergeMap, catchError, of } from 'rxjs';
import { ProjectsServices } from './../../shared/services/projects.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Injectable } from '@angular/core';
import { Project } from 'src/app/shared/models/project/project.model';

@Injectable()
export class ProjectCreateEffects {

  constructor(
    private projectActions: Actions,
    private projectsService: ProjectsServices
  ) {}

  //create project form effect
  createAddProjectFormEffect:Observable<ProjectActionUnion> = createEffect(
    ()=> this.projectActions.pipe(
      ofType((ProjectActionTypes.CREATE_PROJECT_FORM_ADD)),
      map( (projectAction: ProjectActionUnion)=>{
        return new CreateProjectFormAddActionSuccess(projectAction.payload);
      })
    )
  );

  //save project form data effect
  saveProjectFormDataEffect : Observable<ProjectActionUnion> = createEffect(
    ()=> this.projectActions.pipe(
      ofType(ProjectActionTypes.SAVE_PROJECT_FORM_DATA),
      mergeMap( (action: ProjectActionUnion) =>{
        return this.projectsService.createProject(action.payload).pipe(
          map((project: Project)=> new SaveProjectFormDataActionSuccess(project)),
          catchError(err=>of(new SaveProjectFormDataActionError(err.message)))
        )
      })
    )
  );

}
