import { Project } from './../../shared/models/project/project.model';
import { ProjectActionTypes } from './project.action.types';
import { ProjectActionUnion, GetAllProjectsActionSuccess, GetAllProjectsActionError, GetProjectWithUserAssociatedActionSuccess, GetProjectWithUserAssociatedActionError, GetProjectWithCompanyAssociatedActionSuccess, GetProjectWithCompanyAssociatedActionError } from './projects.actions';
import { Observable, mergeMap, map, catchError, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { ProjectsServices } from './../../shared/services/projects.service';
import { Injectable } from '@angular/core';

@Injectable()
export class ProjectGetEffects {
  constructor(
    private projectsService: ProjectsServices,
    private projectActions: Actions
  ) {}

  //get all projects
  getAllProjectsEffect: Observable<ProjectActionUnion> = createEffect(
    () => this.projectActions.pipe(
      ofType(ProjectActionTypes.GET_ALL_PROJECTS),
      mergeMap((action: ProjectActionUnion)=>{
        return this.projectsService.getAllProjects().pipe(
          map( (projects: Project[] )=> new GetAllProjectsActionSuccess(projects)),
          catchError( err => of(new GetAllProjectsActionError(err.message)))
        );
      })
    )
  );

  //get project with user associated
  getProjectWithUserAssociated: Observable<ProjectActionUnion> = createEffect(
    () => this.projectActions.pipe(
      ofType(ProjectActionTypes.GET_PROJECT_WITH_USER_ASSOCIATED),
      mergeMap( (action: ProjectActionUnion)=>{
        return this.projectsService.getProjectWithUserAsociated().pipe(
          map((listAny:any)=> new GetProjectWithUserAssociatedActionSuccess(listAny)),
          catchError(err => of(new GetProjectWithUserAssociatedActionError(err.message)))
        )
      })
    )
  );

  //get project with company associated
  getProjectWithCompanyAssociated: Observable<ProjectActionUnion> = createEffect(
    () => this.projectActions.pipe(
      ofType(ProjectActionTypes.GET_PROJECT_WITH_COMPANY_ASSOCIATED),
      mergeMap( (action: ProjectActionUnion)=>{
        return this.projectsService.getProjectWithCompanyAssociated().pipe(
          map((listAny:any)=> new GetProjectWithCompanyAssociatedActionSuccess(listAny)),
          catchError(err => of(new GetProjectWithCompanyAssociatedActionError(err.message)))
        )
      })
    )
  );
}
