import { Project } from '../../shared/models/project/project.model';
import { ProjectActionTypes } from './project.action.types';
import {
  ProjectActionUnion,
  CreateProjectFormUpdateActionSuccess,
  CreateProjectFormUpdateActionError,
  UpdateProjectWithFormDataActionSuccess,
  UpdateProjectWithFormDataActionError,
  DeleteProjectActionSuccess,
  DeleteProjectActionError,
} from './projects.actions';
import { Observable, mergeMap, map, catchError, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { ProjectsServices } from '../../shared/services/projects.service';
import { Injectable } from '@angular/core';

@Injectable()
export class ProjectUpdateEffects {
  constructor(
    private projectsService: ProjectsServices,
    private projectActions: Actions
  ) {}

  //create form filled with current data to update
  createUpdateFormEffect: Observable<ProjectActionUnion> = createEffect(() =>
    this.projectActions.pipe(
      ofType(ProjectActionTypes.CREATE_PROJECT_FORM_UPDATE),
      mergeMap((action: ProjectActionUnion) => {
        return this.projectsService.getProjectById(action.payload.id).pipe(
          map(
            (project: Project) =>
              new CreateProjectFormUpdateActionSuccess(project)
          ),
          catchError((err) =>
            of(new CreateProjectFormUpdateActionError(err.message))
          )
        );
      })
    )
  );

  //update project with form data
  updateProjectWithFormDataEffect: Observable<ProjectActionUnion> =
    createEffect(() =>
      this.projectActions.pipe(
        ofType(ProjectActionTypes.UPDATE_PROJECT_WITH_FORM_DATA),
        mergeMap((action: ProjectActionUnion) => {
          return this.projectsService.updateProject(action.payload).pipe(
            map(
              (project: Project) =>
                new UpdateProjectWithFormDataActionSuccess(project)
            ),
            catchError((err) =>
              of(new UpdateProjectWithFormDataActionError(err.message))
            )
          );
        })
      )
    );

  deleteProjectEffect: Observable<ProjectActionUnion> = createEffect(() =>
    this.projectActions.pipe(
      ofType(ProjectActionTypes.DELETE_PROJECT),
      mergeMap((action: ProjectActionUnion) => {
        return this.projectsService.deleteProject(action.payload.id).pipe(
          map(() => new DeleteProjectActionSuccess(action.payload)),
        );
      })
    )
  );
}
