import { UsersServices } from '../../shared/services/users.service';
import {
  CreateFormUserAddActionError,
  CreateFormUserAddActionSuccess,
  SaveUserFormAddActionError,
  SaveUserFormAddActionSuccess,
  UserActionUnion,
} from './users.actions';
import { Injectable } from '@angular/core';
import { catchError, map, mergeMap, Observable, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { UserActionTypes } from './user.action.types';
import { User } from 'src/app/shared/models/user/user.model';

@Injectable()
export class UserCreateEffects {
  constructor(
    private userActions: Actions,
    private usersService: UsersServices
  ) {}

  //create a user add form effect
  createAddUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.CREATE_FORM_ADD_USER),
      map((userAction: UserActionUnion) => {
        return new CreateFormUserAddActionSuccess(userAction.payload);
      }),
      catchError((err) => of(new CreateFormUserAddActionError(err.message)))
    )
  );

  //saving user form data effect

  saveUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.SAVE_USERFORM_ADD),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.createUser(userAction.payload).pipe(
          map((user: User) => new SaveUserFormAddActionSuccess(user)),
          catchError((err) => of(new SaveUserFormAddActionError(err.message)))
        );
      })
    )
  );
}
