import { Router } from '@angular/router';
import { UsersServices } from '../../shared/services/users.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Injectable } from '@angular/core';
import {
  GetAllUsersActionError,
  GetAllUsersActionSuccess,
  GetUser_S_ProjectsError,
  GetUser_S_ProjectsSuccess,
  UserActionUnion,
} from './users.actions';
import { Observable, mergeMap, map, catchError, of, tap } from 'rxjs';
import { User } from 'src/app/shared/models/user/user.model';
import { UserActionTypes } from './user.action.types';

@Injectable()
export class UserGetSearchEffects {
  constructor(
    private userActions: Actions,
    private usersService: UsersServices,
    private router: Router
  ) {}

  //getAllUsers effect

  getAllUsersEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.GET_ALL_USERS),
      mergeMap(() => {
        return this.usersService.getAllUsers().pipe(
          map((users: User[]) => new GetAllUsersActionSuccess(users)),
          catchError((err) => of(new GetAllUsersActionError(err.message)))
        );
      })
    )
  );


  // navigate to another component effect
  navigate: Observable<UserActionUnion> = createEffect(
    () =>
      this.userActions.pipe(
        ofType(UserActionTypes.NAVIGATE_TO_USER_S_PROJECTS_COMPONENT_SUCCESS),
        tap(() => {
          this.router.navigate(['/user-s-projects']);
        })
      ),
    { dispatch: false }
  );

  // get user's projects

  loadUserSProjectsEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.GET_USER_S_PROJECTS),
      mergeMap((action: UserActionUnion) => {
        return this.usersService.getUser_s_Projects().pipe(
          map((users: any) => new GetUser_S_ProjectsSuccess(users)),
          catchError((err) => of(new GetUser_S_ProjectsError(err.message)))
        );
      })
    )
  );
}
