import {
  UserActionUnion,
  CreareFormUserEditActionSuccess,
  CreareFormUserEditActionActionError,
  DeleteUserActionSuccess,
  DeleteUserActionError,
  UpdateUserFormActionError,
  UpdateUserFormActionSuccess,
} from './users.actions';
import { map, mergeMap, Observable, catchError, of } from 'rxjs';
import { UsersServices } from '../../shared/services/users.service';
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { User } from 'src/app/shared/models/user/user.model';
import { UserActionTypes } from './user.action.types';
import { Router } from '@angular/router';

@Injectable()

export class UserUpdateEffects {
  constructor(
    private usersService: UsersServices,
    private effectActions: Actions
  ) {}


  // create an update user form effect and fill it with data of a user to update

  createUpdateUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionTypes.CREATE_FORM_USER_EDIT),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.getUserById(userAction.payload.id).pipe(
          map((user: User) => new CreareFormUserEditActionSuccess(user)),
          catchError((err) =>
            of(new CreareFormUserEditActionActionError(err.message))
          )
        );
      })
    )
  );

  // update a user with data of form effect

  updateUserFormDataEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionTypes.UPDATE_USER_FORM),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.updateUser(userAction.payload).pipe(
          map((user: User) => new UpdateUserFormActionSuccess(user)),
          catchError((err) => of(new UpdateUserFormActionError(err.message)))
        );
      })
    )
  );

  //deleteUser effect

  deleteUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionTypes.DELETE_A_USER),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.deleteUser(userAction.payload.id).pipe(
          map(() => new DeleteUserActionSuccess(userAction.payload)),
          catchError((err) => of(new DeleteUserActionError(err.message)))
        );
      })
    )
  );

}
