import { Router } from '@angular/router';
import { Component } from '@angular/core';
import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { Observable } from 'rxjs';
import { map, shareReplay } from 'rxjs/operators';
import { GetAllUsersAction } from 'src/app/ngrx/users/users.actions';
import { Store } from '@ngrx/store';
import { UserState } from 'src/app/ngrx/users/user.state';

@Component({
  selector: 'app-base-nav-menu',
  templateUrl: './base-nav-menu.component.html',
  styleUrls: ['./base-nav-menu.component.css'],
})
export class BaseNavMenuComponent {
  isHandset$: Observable<boolean> = this.breakpointObserver
    .observe(Breakpoints.Handset)
    .pipe(
      map((result) => result.matches),
      shareReplay()
    );

  constructor(
    private breakpointObserver: BreakpointObserver,
    private router: Router,
    private store: Store<{ userReducerKey: UserState }>,
  ) {}

  onUserCreate(): void {
    this.router.navigateByUrl('/user-create');
  }

  onUsersGetAll(){
    this.store.dispatch(new GetAllUsersAction({}));
  }
}
