import { MsgState } from 'src/app/ngrx/msg.state';
import { DeleteCompanyAction, GetAllCompaniesAction } from './../../../../ngrx/companies/companies.actions';
import { CompanyState } from './../../../../ngrx/companies/company.state';
import { Router } from '@angular/router';
import { Component, Input, OnInit } from '@angular/core';
import { Company } from 'src/app/shared/models/company/company.model';
import { Store } from '@ngrx/store';

@Component({
  selector: 'app-companies-list',
  templateUrl: './companies-list.component.html',
  styleUrls: ['./companies-list.component.css'],
})
export class CompaniesListComponent implements OnInit {
  constructor(
    private router: Router,
    private store: Store<{ companyReducerKey: CompanyState }>
  ) {}

  @Input() inputCompaniesList!: Company[];
  msgState = MsgState;

  ngOnInit(): void {}

  onCompanyEdit(company: Company) {
    this.router.navigate(['/company-update/' + company.id]);
  }

  onCompanyDelete(company: Company) {
    if (window.confirm(this.msgState.CONFIRM_DEL)) {
      this.store.dispatch(new DeleteCompanyAction(company));
    }
  }

  companyName!: string;

  onSearch() {
    if(this.companyName==''){
      this.store.dispatch(new GetAllCompaniesAction({})); //to refresh list
    }
    else{
      this.inputCompaniesList = this.inputCompaniesList.filter(
        (company: Company) => {
          return company.companyName
            .toLocaleLowerCase()
            .match(this.companyName.toLocaleLowerCase());
        }
      );
    }
  }
}
