import { SaveCompanyFormDataAddAction } from './../../../../ngrx/companies/companies.actions';
import { Company } from './../../../../shared/models/company/company.model';
import { CompaniesServices } from './../../../../shared/services/companies.service';
import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { CreateCompanyFormAddAction } from 'src/app/ngrx/companies/companies.actions';
import { CompanyState } from 'src/app/ngrx/companies/company.state';
import { MsgState } from 'src/app/ngrx/msg.state';
import { CompanyType } from 'src/app/shared/models/company/company.type';

@Component({
  selector: 'app-company-create',
  templateUrl: './company-create.component.html',
  styleUrls: ['./company-create.component.css']
})
export class CompanyCreateComponent implements OnInit {

  constructor(
    private store: Store<{ companyReducerKey: CompanyState }>,
    private fb: FormBuilder,
    private datePipe: DatePipe,
    private router: Router,
    private companiesService: CompaniesServices
  ) {}

  companyState!: CompanyState;
  companyFormGroup!: FormGroup;
  msgState = MsgState;
  compTypes = CompanyType;

  ngOnInit(): void {
    this.store.dispatch(new CreateCompanyFormAddAction());
    this.store.subscribe( (state)=>{
      this.companyState = state.companyReducerKey;

      this.companyFormGroup = this.fb.group({
        id: [0, Validators.required],
        companyName:['', [Validators.required, Validators.minLength(5), Validators.maxLength(20)]],
        companyType:['', Validators.required],
        creationDate:[this.datePipe.transform(new Date(),'MM dd yyyy')]
      })
    });
  }


  onCompanyCreate(){
     let company: Company = this.companyFormGroup.value;
     this.companiesService.getCompanyByCompanyName(company.companyName).subscribe(
       (companies: Company[]) => {
         if( companies.length > 0){
           alert(this.msgState.EXIST);
           this.store.dispatch(new CreateCompanyFormAddAction());
         }
         else{
           if(window.confirm(this.msgState.CONFIRM_ADD)){
            this.store.dispatch(new SaveCompanyFormDataAddAction(company));
           }
         }
       }
     )

  }
  onSaveOk(){
    this.router.navigateByUrl('/companies-management');
  }

}
