import { MsgState } from 'src/app/ngrx/msg.state';
import { UsersServices } from '../../../../shared/services/users.service';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { CreareFormUserEditAction, UpdateUserFormAction } from '../../../../ngrx/users/users.actions';
import { UserState } from 'src/app/ngrx/users/user.state';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Store } from '@ngrx/store';

@Component({
  selector: 'app-user-update',
  templateUrl: './user-update.component.html',
  styleUrls: ['./user-update.component.css'],
})
export class UserUpdateComponent implements OnInit {
  userId: number;
  userFormGrp!: FormGroup;
  userState!: UserState;

  constructor(
    activatedRoute: ActivatedRoute,
    private usersService: UsersServices,
    private store: Store<{ userReducerKey: UserState }>,
    private fb: FormBuilder,
    private router: Router
  ) {
    this.userId = activatedRoute.snapshot.params['id'];
  }

  readonly msg = MsgState;

  ngOnInit(): void {
    this.usersService.getUserById(this.userId).subscribe((user) => {
      this.store.dispatch(new CreareFormUserEditAction(user));

      this.store.subscribe((state) => {
        this.userState = state.userReducerKey;

        this.userFormGrp = this.fb.group({
          id: [this.userState.currentUser?.id, Validators.required],
          firstname: [
            this.userState.currentUser?.firstname,
            [
                Validators.required,
                Validators.minLength(3),
                Validators.maxLength(15),
            ],
            ],
            lastname: [
              this.userState.currentUser?.lastname,
              [
                Validators.required,
                Validators.minLength(3),
                Validators.maxLength(15),
              ],
            ],
            email: [
              this.userState.currentUser?.email,
              [Validators.required, Validators.email],
            ],
            address: this.fb.group({
              numRue: [
                this.userState.currentUser?.address.numRue,
                [Validators.required, Validators.min(1)],
              ],
              nomRue: [
                this.userState.currentUser?.address.nomRue,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(50),
                ],
              ],
              bp: [
                this.userState.currentUser?.address.bp,
                [Validators.required, Validators.min(1)],
              ],
              ville: [
                this.userState.currentUser?.address.ville,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
              pays: [
                this.userState.currentUser?.address.pays,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
            }),
          });

      });
    });
  }

  onUserUpdate() {
    if(window.confirm(MsgState.CONFIRM_ADD)){
      this.store.dispatch(new UpdateUserFormAction(this.userFormGrp.value));
    }
  }

  onUpdateOk(){
    this.router.navigateByUrl('/users-management')
  }
}
