import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { catchError, Observable, throwError } from 'rxjs';
import { environment } from 'src/environments/environment';
import { User } from '../models/user/user.model';

@Injectable({
  providedIn: 'root',
})
export class UsersServices {
  constructor(private http: HttpClient) {}

  handleError(err: HttpErrorResponse) {
    if (err.status === 0) {
      console.error('error has occurred:', err.error);
    } else {
      console.error(
        'Backend returned code ${error.status}, body was:',
        err.error
      );
    }

    return throwError(() => {
      new Error('something went wrong, try again');
    });
  }

  baseUrl: string = Math.random() > 0.01 ? environment.host : environment.unreachibleHost;
  headers = { 'content-type': 'application/json' };

  getAllUsers(): Observable<User[]> {
    let allUsers$: Observable<User[]> = this.http.get<User[]>(
      this.baseUrl + '/users'
    );

    return allUsers$;
  }

  getUser_s_Projects(): Observable<any> {
    return this.http
      .get<any>(this.baseUrl + '/users?_embed=projects')
      .pipe(catchError(this.handleError));
  }

  createUser(user: User): Observable<User> {
    return this.http
      .post<User>(this.baseUrl + '/users', user, { headers: this.headers })
      .pipe(catchError(this.handleError));
  }

  deleteUser(userId: number) {
    return this.http
      .delete(this.baseUrl + '/users/' + userId, {
        headers: this.headers,
      })
      .pipe(catchError(this.handleError));
  }

  getUserById(userId: number): Observable<User> {
    return this.http.get<User>(this.baseUrl + '/users/' + userId);
  }

  getUserByName(lastname: string): Observable<User[]>{
    return this.http.get<User[]>(this.baseUrl+'/users?lastname='+lastname);
  }

  updateUser(user: User): Observable<User> {
    return this.http
      .put<User>(this.baseUrl + '/users/' + user.id, user, {
        headers: this.headers,
      })
      .pipe(catchError(this.handleError));
  }

  searchUser(firstnameKeyword: string): Observable<User[]> {
   let users = this.http.get<User[]>(
      this.baseUrl + '/users?firstname_like=' + firstnameKeyword
    );

    return users;

  }
}
